# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka, Miroslav Bursa
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

"""
LoadWriteData
---------------

Load and write data in several formats.
    * text files
    * matlab files
    * csv files [not supported yet]
    txt: mb: loading csv directly from TXT/CSV file generated from BDI

Reference
~~~~~~~~~

.. autoclass:: EnumVariableName
.. autoclass:: LoadData
   :members:
"""

# Import global modules
import numpy as np
import logging
import os
import struct
from scipy.io import loadmat
#import scipy.io as sio
import csv
# local imports
import Common
from Config import ConfigStatic


class EnumVariableName:
    """
    Class EnumVariableName contains description and names of variables in a matlab file that can be used in python

    Examples of variables in mat-file (others not supported yet):

    :param fhr: fetal heart rate signal
    :param uc: uterine contractions signal
    :param timestamp: time stamp giving samples that are valid
    :type fhr: numeric vector
    :type uc: numeric vector
    :type timestamp: numeric vector

    | Example of matlab file:
    | fhr = [150,150,151,152,150]
    | uc = [7,10,10,9,8]
    | timestamp = [1,2,3,10,11] % timestamp represents timesamples; that is during time: 4,5,6,7,8,9 there is no recording.
    | Do not substitute with artifacts! The timestamp is used for situations when the recording was stopped and started again.

    save('temp.mat','fhr','uc','timestamp')

    | View file in ctgViewerLite
    | 1) open via menu bar in ctgViewerLite
    | 2) call ctgViewerLite from command prompt, see REFERENCE NEEDED: for details
    | 3) call ctgViewerLite from matlab, see REFERENCE NEEDED: for details
    """

    fhr = 'fhr'              # required
    uc = 'uc'                # required
    timestamp = 'timestamp'  # required
    sBaseline = 'baseline'   # optional
    time_samp = 'time_samp'  # optional
    time_ms = 'time_ms'      # optional
    fs = 'fs'                # optional - sampling frequency


class LoadData:
    """
    Load data from various types of files
    """

    def __init__(self):
        self._log = logging.getLogger(ConfigStatic.logger_name)

    def readPhysionetHeader(self, inFile):
        """
        Read a file header in physionet format

        :param inFile: filename to read
        :type inFile: string
        :return lHeader
        :rtype lHeader list of dict
        """
        if Common.file_exists(inFile) == False:
            raise IOError('File not exists: {0}'.format(inFile))

        self._log.info('Loading file {0}'.format(inFile))

        f = open(inFile,'r')

        firstLine = f.readline()
        A = firstLine.split(' ')

        #fileName = A[0]
        nNrSignals = int(A[1])
        nFs = int(A[2])
        nNrSamples = int(A[3])

        lHeader = list() # save information to a list
        for dummy in range(nNrSignals):

            dictHeader = dict()
            A = f.readline()
            A = A.split(' ')
            dictHeader['name'] = A[0]
            dictHeader['format'] = int(A[1])

            # get type of signal (either FHR or UX)
            sType = str(A[8])
            n = len(sType)
            sType = sType[0:n-1]

            s = A[2]
            if sType == 'FHR':
                idx = s.index('(')
                dictHeader['gain'] = int(s[0:idx])
            elif sType == 'UC':
                idx = s.index('/')
                dictHeader['gain'] = int(s[0:idx])

            dictHeader['bitres'] = int(A[3])
            dictHeader['zerovalue'] = int(A[4])
            dictHeader['firstvalue'] = int(A[5])
            dictHeader['checksum'] = int(A[6])

            dictHeader['fs'] = nFs

            lHeader.append(dictHeader)


        clinInfo = dict()
        line = f.readline()
        while line:
            if not line == '\n':
                if not line[1] == '-':
                    param = line[1:13]
                    param = param.strip()
                    param = self.__removeSpecificChar(param)

                    try:
                        if param == 'Sig2Birth':
                            value = self.__parseNumber(line[14:len(line)])
                        else:
                            value = self.__parseNumber(line[14:-1])
                    except Exception,msg:
                        if param == 'BDecf' or param == 'BE' or param == 'pCO2':
                            value = np.nan
                        else:
                            self._log.error('Parse int error: param: {0}, msg:{1}'.format(param,msg))

                    clinInfo[param] = value

            line = f.readline()

        lHeader.append(clinInfo)
        f.close()
        return lHeader,nNrSignals,nFs,nNrSamples

    def readPhysionetHeaderSignalName(self, inFile):
        """
        Read a physionet header when the name of signal is provided

        :param inFile: filename of signal, which header is desired
        :type inFile: string
        :return list of parameters
        """
        path, name_all = os.path.split(inFile)
        name, dummy = os.path.splitext(name_all)
        headerFile = os.path.join(path,name + ".hea")
        return self.readPhysionetHeader(headerFile)

    def readPhysionetSignal16(self, inFile, lHeader=None):
        """ read a signal in physionet format, only format 16 is supported """

        if Common.file_exists(inFile) == False:
            raise IOError('File not exists: {0}'.format(inFile))

        if lHeader is None:
            # read a header if it was not supplied
            lHeader,nNrSignals,dummy,nNrSamples = self.readPhysionetHeaderSignalName(inFile)

        fhrHead = lHeader[0]
        tocoHead = lHeader[1]
        if not fhrHead['format'] == 16:
            raise IOError("Error trying to read signal in different format!")

        f = open(inFile,'rb')

        data_packed = f.read()

        nNr = len(data_packed)/2 # 2 does not mean two signals
        myfmt='H'*(nNr) # H means uint16

        if not nNr/2 == nNrSamples:
            self._log.error("Error number of samples incorrect")

        if not len(data_packed) == struct.calcsize(myfmt):
                print len(data_packed)
                print struct.calcsize(myfmt)
                self._log.error("Invalid argument for reading binary file")

        data = struct.unpack(myfmt, data_packed) # note the ',' in 'value,': unpack apparently returns a n-uple

        # init numpy array
        aDataTemp = np.zeros((nNrSamples,nNrSignals+1),np.float)

        cnt = 0
        for i in range(0,nNr,2):
            aDataTemp[cnt,0] = float(data[i])/fhrHead['gain']
            aDataTemp[cnt,1] = float(data[i+1])/tocoHead['gain']
            aDataTemp[cnt,2] = cnt
            cnt += 1

        aDataAll = dict()
        aDataAll[EnumVariableName.fhr] = aDataTemp[:,0]
        aDataAll[EnumVariableName.uc] = aDataTemp[:,1]
        aDataAll[EnumVariableName.time_samp] = aDataTemp[:,2]
        aDataAll[EnumVariableName.timestamp] = aDataAll[EnumVariableName.time_samp]

        return aDataAll, lHeader

    def readBdiTxtFile(self, inFile):
        """
        load data from txt file (generated from BDI file), the structure of csv is expected as:
        timestamp, fhr, uc (fields: 1, 4, 11)
        """
        f = open(inFile,'r')

        #count no_lines: not very elegant...
        nNrSamples = 0
        for dummy in range(0,11):
            f.next()

        for line in f:
            if not line[:4] == '****':
                nNrSamples += 1

        f.seek(0)
        for dummy in range(0,11):
            f.next()

        count = 0
        timestamp = np.zeros((nNrSamples,1), np.int)
        data = np.zeros((nNrSamples,2), np.float)

        for line in f:
            if not line[:4] == '****':
                row = line.split(',')
                #print row
                timestamp[count] = int(row[0].strip())
                data[count,0] = float(row[3].strip())
                data[count,1] = float(row[10].strip())
                if not data[count,0] == 0.0:
                    data[count,0] =  data[count,0] /4;

#              if count == 1024:
#                  print data[count,0],data[count,1],timestamp[count]
                count += 1

        aDataAll = dict()
        aDataAll[EnumVariableName.timestamp] = timestamp
        aDataAll[EnumVariableName.fhr] = data[:,0]
        aDataAll[EnumVariableName.uc] = data[:,1]

        f.close()

        return aDataAll, 0

    def readCsvFile(self, inFile):
        """
        load data from csv file, the structure of csv is expected as:
        timestamp, fhr, uc
        """
        f = open(inFile,'r')

        nNrSamples = Common.get_number_lines_in_stream(f)
        fileContent = csv.reader(f, delimiter=',')

        bHasHeader = csv.Sniffer().has_header(f.read(1024))
        f.seek(0) # return to begin

        if bHasHeader:
            nNrSamples -= 1

        timestamp = np.zeros((nNrSamples,1),np.int)
        data = np.zeros((nNrSamples,2),np.float)
        count = 0

        for row in fileContent:
            #print row
            if count == 0 and bHasHeader == True:
                count = 0
                bHasHeader = False
                continue

            timestamp[count] = int(row[0])
            data[count,0] = float(row[1])
            data[count,1] = float(row[2])
            count += 1

        aDataAll = dict()
        aDataAll[EnumVariableName.timestamp] = timestamp
        aDataAll[EnumVariableName.fhr] = data[:,0]
        aDataAll[EnumVariableName.uc] = data[:,1]

        f.close()

        return aDataAll, 0

    def writeCsvFile(self, inFile, data, sHeader = None):
        """
        write data into csv file.
        the function is not using the python csv module because of unable to setup float precision
        """
        f = open(inFile,'w+')

        # write header
        if sHeader is None:
            sHeader = '{0},{1},{2}\n'.format(EnumVariableName.timestamp,EnumVariableName.fhr,EnumVariableName.uc)

        f.write(sHeader)

        timestamp = data[EnumVariableName.timestamp]
        fhr = data[EnumVariableName.fhr]
        uc = data[EnumVariableName.uc]

        nNrSamples = len(timestamp)
        #print nNrSamples

        for i in range(0,nNrSamples):
            s = '{0},{1},{2}\n'.format(int(timestamp[i]),round(fhr[i],2),round(uc[i],2))
            f.write(s)

        f.close()
        return data


    def readMatlabFile(self, sFile):
        """ read a signal in matlab format, for specification of matlab file see :py:class:`EnumVariableName` """

        if Common.file_exists(sFile) == False:
            raise IOError('File not exists: {0}'.format(sFile))

        data = loadmat(sFile)

        if not EnumVariableName.fhr in data:
            raise Exception("The variable 'fhr' must be present in the file: {0}".format(sFile))

        if not EnumVariableName.uc in data:
            raise Exception("The variable 'uc' must be present in the file: {0}".format(sFile))

        if not EnumVariableName.timestamp in data:
            raise Exception("The variable 'timestamp' must be present in the file: {0}".format(sFile))

        if not EnumVariableName.fs in data:
            data[EnumVariableName.fs] = 4*np.ones(1, np.int)

        # the data from matfile were loaded in numpy arrays in a different shape, we want to get rid of it
        for s in data.keys():
            if not s[0] == '_': # if the variable is global var.
                t = data[s]
                if t.dtype == np.int: # is it is not numpy array
                    continue

                data[s] = t[:,0]

        #print fhr[:,0]

        nNrSamples = len(data[EnumVariableName.fhr])
        data[EnumVariableName.time_samp] = np.linspace(1, nNrSamples, nNrSamples)
        #data[EnumVariableName.tim]

        return data,0

    def __parseNumber(self, sValue = None):

        if sValue.find('.') > 0:
            return float(sValue)
        else:
            return int(sValue)

    def __removeSpecificChar(self,s):

        s = s.replace(' ', '_')
        s = s.replace('.', '')
        s = s.replace('(', '_')
        s = s.replace(')', '')
        s = s.replace('/', '_')

        return s



if __name__ == '__main__':

    #file1 = 'files/1001.mat'
    file1 = '/tmp/tpa1fb9b57_683a_4f3c_954f_5e829bd2578a.mat'
    data_loader = LoadData()
    #data_loader.readPhysionetSignal16(file1)
    #data_loader.readPhysionetHeader(file1)
    data_loader.readMatlabFile(file1)

