# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

"""
MainWindow
----------------

The MainWindow module is the main class that provides interaction between gui and user.
This class integrates all other functionality.

Reference
~~~~~~~~~~~
.. autoclass:: Main
   :members:
"""

DEBUG_PROFILE = False

# system imports
import sys
import os
from PyQt4 import QtCore, QtGui
import logging
import argparse

# local imports
from MainWindowUI import Ui_MainWindow
from GuiForms import AboutDialog, ConvertFileForm, DownoladDbForm
from Logger import Logger
from Config import ConfigStatic, EnumIniVar, ConfigIni
from LoadWriteData import LoadData
from LoadWriteData import EnumVariableName as EnumVarName
from ClinInfo import ClinInfo
import Common


#conditional import
if DEBUG_PROFILE:
    import cProfile
    import pstats

class Main(QtGui.QMainWindow):

    def __init__(self, args = sys.argv):
        QtGui.QMainWindow.__init__(self)

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self._log = logging.getLogger(ConfigStatic.logger_name)
        self._log.info('passed')
        self._log.info('profiling: {0}'.format(DEBUG_PROFILE))

        self._dataLoader = LoadData()

        self._createConnections()

        self._configIni = ConfigIni()
        self._convertFilesForm = ConvertFileForm()
        self._downloadDbForm = DownoladDbForm()

        # dock widgets
        self._clinInfoWidget = ClinInfo()
        self._clinInfoWidget.clearAll()
        self.ui.dockClinInfo.setWidget(self._clinInfoWidget)
        self.ui.dockClinInfo.setVisible(self._configIni.get_var(EnumIniVar.dockClinInformationVisible))

        #self.ui.dockClinInfo.dockWidgetArea

        self._processCmdArgs(args)

    def _createConnections(self):
        """ Creates connections in MainWindow """

        # file menu
        self.connect(self.ui.actionQuit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))
        self.connect(self.ui.actionOpen, QtCore.SIGNAL('triggered()'), self._openFile)
        self.connect(self.ui.actionAbout, QtCore.SIGNAL('triggered()'), self._showAbout)
        self.connect(self.ui.actionConvert_files, QtCore.SIGNAL('triggered()'), self._showConvertFiles)
        self.connect(self.ui.actionDownload_CTU_UHB_data, QtCore.SIGNAL('triggered()'), self._showDownloadDB)
        self.connect(self.ui.actionAttribute_selection, QtCore.SIGNAL('triggered()'), self._showAttributeSelection)
        #self.connect(self.ui.actionClose, QtCore.SIGNAL('triggered()'), self.closeFile)

        self.connect(self.ui.actionClinical_information, QtCore.SIGNAL('triggered()'),self._dockWidgetsToggle)
        self.connect(self.ui.dockClinInfo, QtCore.SIGNAL("visibilityChanged(bool)"),self._dockWidgetVisibility)


    def _dockWidgetVisibility(self):
        """ visibility changed, set the property of checkbox"""
        self.ui.actionClinical_information.setChecked(self.ui.dockClinInfo.isVisible())

    def _dockWidgetsToggle(self):
        """ Set docks widget visible or invisible  """
        if (self.ui.actionClinical_information.isChecked()  == True):
            self.ui.dockClinInfo.show()
        else:
            self.ui.dockClinInfo.hide()


    def _openFile(self):
        """ Open file in gui. Files are filtered based on extension """
        file1 = QtGui.QFileDialog.getOpenFileName(self, 'Open File', self._configIni.get_var(EnumIniVar.lastUsedDirFiles), 'Files (*.dat *.csv *.mat *.txt)');

        if file1 == "":
            self._log.debug("File name is empty - no file selected")
            return -1

        file1 = str(file1)
        dummy,ext = Common.get_filename_and_ext(file1)

        if ext == '.dat':
            self.plotPhysionetFile(file1)
        elif ext == '.mat':
            self.plotMatlabFile(file1)
        elif ext == '.csv':
            self.plotCsvFile(file1)
        elif ext == '.txt':
            self.plotBdiTxtFile(file1)

        self._configIni.set_var(EnumIniVar.lastUsedDirFiles,os.path.dirname(file1))
        return 0

    def plotPhysionetFile(self, sFile):

        try:
            aData, lHeader = self._dataLoader.readPhysionetSignal16(sFile)
        except Exception as ex:
            self._log.error(ex)
            return

        sig = lHeader[0]
        dictClinInfo = lHeader[2]
        nFs = sig['fs']
        self.ui.PlotWidget.setSamplingFreqAllPlots(nFs)

        nNrSamples = len(aData[EnumVarName.time_samp])
        timeString = self.ui.PlotWidget.fhrPlot.samples2time(nNrSamples, nFs)
        self.ui.PlotWidget.plot(aData[EnumVarName.time_samp], aData[EnumVarName.fhr], aData[EnumVarName.uc], timeString)

        # clinical information
        self._clinInfoWidget.setAll(dictClinInfo)

    def plotMatlabFile(self, sFile):

        try:
            aData,dummy = self._dataLoader.readMatlabFile(sFile)
        except Exception as ex:
            self._log.error(ex)
            return

        #nFs = aData[EnumVarName.fs]
        aFhr = aData[EnumVarName.fhr]
        aUc =  aData[EnumVarName.uc]
        #aTime_samp = aData[EnumVarName.time_samp]
        aTimestamp = aData[EnumVarName.timestamp]

        #fs = aData[EnumVarName.fs]
        fs = 10
        self.ui.PlotWidget.setSamplingFreqAllPlots(fs)

        nNrSamples = len(aFhr)
        timeString = self.ui.PlotWidget.fhrPlot.samples2time(nNrSamples, fs)
        self.ui.PlotWidget.plot(aTimestamp, aFhr, aUc, timeString)
        #self.ui.PlotWidget.plot(aTimestamp, aFhr, aUc)

        # clinical information
        #self._clinInfoWidget.setAll(dictClinInfo)

    def plotCsvFile(self, sFile):

        try:
            aData,dummy = self._dataLoader.readCsvFile(sFile)
        except Exception as ex:
            self._log.error(ex)
            return

        aFhr = aData[EnumVarName.fhr]
        aUc =  aData[EnumVarName.uc]
        aTimestamp = aData[EnumVarName.timestamp]

        self.ui.PlotWidget.plot(aTimestamp, aFhr, aUc)

    def plotBdiTxtFile(self, sFile):

        try:
            aData,dummy = self._dataLoader.readBdiTxtFile(sFile)
        except Exception as ex:
            self._log.error(ex)
            return

        aFhr = aData[EnumVarName.fhr]
        aUc =  aData[EnumVarName.uc]
        aTimestamp = aData[EnumVarName.timestamp]

        self.ui.PlotWidget.plot(aTimestamp, aFhr, aUc)


    def _showAbout(self):
        win = AboutDialog()
        win.exec_()

    def _showConvertFiles(self):
        self._convertFilesForm.show()

    def _showAttributeSelection(self):

        msg = QtGui.QMessageBox()
        msg.setText('Not implemented yet.')
        msg.setIcon(QtGui.QMessageBox.Warning)
        msg.setStandardButtons(QtGui.QMessageBox.Ok);
        msg.exec_()

    def _showDownloadDB(self):
        self._downloadDbForm.show()

    def closeEvent(self, event):
        self._configIni.set_var(EnumIniVar.dockClinInformationVisible, self.ui.dockClinInfo.isVisible())
        self._configIni.write_config()

    def _parseCmdArgs(self, args):
        parser = argparse.ArgumentParser(description='ctgViewer -- browsing and viewing CTU-UHB database')
        parser.add_argument('-p','--physionet-file', help='Input file in the physionet format', required=False)
        parser.add_argument('-m','--matlab-file', help='Input file in the matlab format', required=False)
        return parser.parse_args()

    def _processCmdArgs(self, args):

        parsed_args = self._parseCmdArgs(args)

        if not parsed_args.physionet_file is None:
            self.plotPhysionetFile(parsed_args.physionet_file)

        if not parsed_args.matlab_file is None:
            self.plotMatlabFile(parsed_args.matlab_file)

        #print parsed_args.physionet_file
        #print parsed_args.matlab_file


def main():

    Logger(ConfigStatic.logger_name)
    app = QtGui.QApplication(sys.argv)
    window = Main(sys.argv)
    window.show()
    window.ui.PlotWidget.updatePlots()

    sys.exit(app.exec_())

if __name__ == '__main__':

    if DEBUG_PROFILE:
        cProfile.run('main()', 'profile_data')
        p = pstats.Stats('profile_data')
        p.sort_stats('time').print_stats(20)
        p.sort_stats('cumulative').print_stats(20)
    else:
        main()
