# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

"""
PyQwtNavigationPlot
--------------------

The navigation plot shows overview of complete signals. By clicking on this
plot the controled plots are moved to the clicked position. The area of visible signal
is highlighted using py:class:´´

Reference
~~~~~~~~~

.. autoclass:: PyQwtNavigationPlot
   :members:
   :undoc-members:

.. autoclass:: PyQwtNavigationPlotHighlight
   :members:
   :undoc-members:

"""
from PyQt4.QtCore import pyqtSignal
from PyQt4.QtGui import QColor
from PyQt4 import Qt
from PyQt4.Qwt5 import QwtPlotCurve, QwtScaleDiv, QwtPlot
from PyQwtWidgetGui import PyQwtWidgetGui
from PyQwtCanvasPicker import PyQwtCanvasPicker

class PyQwtNavigationPlot(PyQwtWidgetGui):
    """
    The navigation plot shows overview of complete signals. By clicking on this
    plot a signal is emitted with clicked point position.

    :param parent:
    :param lstControledPlots: list of controled plots
    :type  parent: QWidget
    :type  lstControledPlots: :py:mod:`PyQwtWidgetGui`
    """

    #selectedPointChanged = pyqtSignal(['int'])

    def __init__(self, parent = None, lstControledPlots = None):
        PyQwtWidgetGui.__init__(self, parent)

        self._lstControledPlots = lstControledPlots
        self._timeString = None
        self._selectedPoint = 0
        self._height = 75

        self.ylabel("FHR")
        self.xlabel("")
        color = QColor(200,200,200)
        #color.setAlpha(200)
        self.setCanvasBackground(color)
        self.xAxisEnabled(True)
        self.yAxisEnabled(False)
        self.setMaximumHeight(self._height)
        self.setMinimumHeight(self._height)

        self.setLocatorMinuteInterval(30)
        self.setYMinMax(50, 210)

        self.canvasPicker = PyQwtCanvasPicker(self)
        self.canvasPicker.pointClicked.connect(self.setSelectedPoint)

        self.signalPlotCleared.connect(self.reinit)
        self.reinit()

    def reinit(self):

        navPlotHighlight = PyQwtNavigationPlotHighlight(self)
        navPlotHighlight.attach(self)

    def setXAxis(self, centerPoint = None):

        if self._bPlotDatesXAxis:
            scaleDiv = self.axisScaleDiv(QwtScaleDiv.MajorTick)
            scaleDiv.setTicks(QwtScaleDiv.MinorTick, [])
            scaleDiv.setTicks(QwtScaleDiv.MediumTick, [])
            scaleDiv.setTicks(QwtScaleDiv.MajorTick, self._timeTics)
            #print scaleDiv.isValid()
            #print self._timeTics
            scaleDiv.setInterval(self._minTime, self.xAxisMaxSample())
            self.setAxisScaleDiv(QwtPlot.xBottom, scaleDiv)
            self.replot()

    def getSelectedPoint(self):
        """
        When user clicked on the plot, this variable holds time of a point

        :rtype: float
        """
        return self._selectedPoint

    def setSelectedPoint(self, point):
        """
        Set point clicked by a user.

        :param point: clicked point
        :type  point: float
        """

        self._selectedPoint = point.x()
        self._selectedPointChanged(self._selectedPoint)

    def _selectedPointChanged(self, selectedPoint):
        """
        When user click on navigation plot:
        Updates all controled plots -> set their x axis
        Updates (this) navigation plot.
        """
        for plot in self._lstControledPlots:
            plot.setXAxis(self._selectedPoint)

        self.replot()
        #self.selectedPointChanged.emit(selectedPoint)

class PyQwtNavigationPlotHighlight(QwtPlotCurve):
    """
    The navigation plot highlight is used for highlighting area where
    user clicked with mouse. This class is inherited from Qwt.QwtPlotCurve.

    :param pNavPlot: navigation plot widget
    :type  lstControledPlots: :py:mod:`PyQwtNavigationPlot`
    """

    def __init__(self, pNavPlot):
        QwtPlotCurve.__init__(self)

        self.__pNavPlot = pNavPlot
        _color = QColor(0,0,0)
        _color.setAlpha(50)
        self._penColor = _color
        self._brushColor = _color

    def drawFromTo(self, painter, xMap, yMap, start, stop):
        """
        Draws rectangles around a point where user clicked

        :param painter:
        :param xMap:
        :param yMap:
        :param start:
        :param stop:
        :type  painter: QPainter
        """
        painter.setPen(Qt.QPen(self._penColor, 1))
        painter.setBrush(self._brushColor)

        maxYView = self.__pNavPlot.viewYMaxSample()
        yMinMarginPx, yMaxMarginPx =  self.__pNavPlot.getMarginsYPx()

        py1 = yMap.transform(0) + yMinMarginPx
        py2 = yMap.transform(maxYView) - yMaxMarginPx

        clickedPoint = self.__pNavPlot.getSelectedPoint()
        viewXMinSamp = self.__pNavPlot._lstControledPlots[0].viewXMinSample()
        viewXMaxSamp = self.__pNavPlot._lstControledPlots[0].viewXMaxSample()

        # check boundaries for line plotting
        if clickedPoint  < viewXMinSamp:
            clickedPoint = viewXMinSamp
        elif clickedPoint  > viewXMaxSamp:
            clickedPoint = viewXMaxSamp

        #drawRect(x,  y, width, height)
        painter.drawRect(xMap.transform(viewXMinSamp), py1, xMap.transform(viewXMaxSamp) - xMap.transform(viewXMinSamp), py2 - py1)
        painter.drawLine(xMap.transform(clickedPoint), py1, xMap.transform(clickedPoint), py2)
        #painter.drawLine(xMap.transform(clickedPoint), yMinMarginPx, xMap.transform(clickedPoint), yMaxMarginPx)
