# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)


"""
PyQwtWidgetGui
----------------

The PyQwtWidgetGui module provides the plotting widget base class. This class ensures:
    * proper ratio of scales cm/min, bpm/cm
    * appearance of all derived plot

Reference
~~~~~~~~~~~

.. autoclass:: PyQwtWidgetGui
   :members:

.. autoclass:: TimeScaleDraw
   :members:

.. autoclass:: EnumPlotSpeed
.. autoclass:: EnumPlotBpm

"""

#from PyQt4 import QtCore, QtGui
from PyQt4 import QtCore, QtGui, Qt

#from PyQt4.QtCore import *
#from PyQt4.QtGui import *
#from PyQt4 import Qt
import PyQt4.Qwt5 as Qwt
#from math import *

import numpy as np
import logging
import sys

DEBUG_PROFILE = False

if DEBUG_PROFILE:
    import cProfile
    import pstats

class EnumPlotSpeed:
    """
    Plot/paper speed enum - defines cm/min ratio.
    """
    step = 1
    min = oneCmMin = 1
    twoCmMin = 2
    max = threeCmMin = 3

class EnumPlotBpm:
    """
    Plot/paper bpm size enum - defines bpm/cm ratio.
    """
    step = 10
    min = twentyBpmCm = 20
    max = thirtyBpmCm = 30

class PyQwtWidgetGui(Qwt.QwtPlot):
    """
    Base class plotting widget based on PyQwt.

    :param parent:
    """
    DEBUG_Y_AXIS_SIZE = False
    DEBUG_X_AXIS_SIZE = False

    signalPlotCleared = QtCore.pyqtSignal()

    def __init__(self, parent = None):
        Qwt.QwtPlot.__init__(self, parent)

        self._log = logging.getLogger("MyLogger")
        self._log.info('passed')

        self._fontApp = QtGui.QFont()

        dpi=96
        self._dpi2px = dpi*(1/2.54)

        self._minView = 0
        self._maxView = 220
        self._minTime = 0
        self._maxTime = 0
        self._centerPoint = 0
        self._samplingFreq = 4

        self._x = 0
        self._y = 0

        self._definePaper()

        self.__canvasXMap = self.canvasMap(Qwt.QwtPlot.xBottom)
        self.__canvasYMap = self.canvasMap(Qwt.QwtPlot.yLeft)

        self.setCanvasBackground(Qt.Qt.white)
        self.setAxisTitle(Qwt.QwtPlot.xBottom, "Time [min]")
        self.setAxisTitle(Qwt.QwtPlot.yLeft, "")

        # curves
        self.curve1 = Qwt.QwtPlotCurve('signal')
        ##self.curve1.setRenderHint(Qwt.QwtPlotItem.RenderAntialiased)
        #self.curve1.setPen(QPen(Qt.Qt.black,0, Qt.Qt.SolidLine))
        #self.curve1.setYAxis(Qwt.QwtPlot.yLeft)
        #self.curve1.attach(self)


        # time scale - for string time scale
        self.timeScale = TimeScaleDraw(self)
        self.setAxisScaleDraw(Qwt.QwtPlot.xBottom, self.timeScale)

        # time scale - for Y axis
        self.yAxisScale = AxisYScaleDraw(self)
        self.setAxisScaleDraw(Qwt.QwtPlot.yLeft, self.yAxisScale)

        self._bPlotDatesXAxis = False
        self._timeTics = list()
        self._locatorMinuteInterval = 5
        self._locatorHourInterval = 0

        # grid
        self.grid = Qwt.QwtPlotGrid()
        self.grid.enableYMin(True)
        self.grid.enableXMin(True)
        self.grid.setMajPen(QtGui.QPen(Qt.Qt.lightGray, 0, Qt.Qt.SolidLine))
        self.grid.setMinPen(QtGui.QPen(Qt.Qt.lightGray, 0, Qt.Qt.DotLine))
        self.grid.attach(self)

        self.__setAxisAppFont()

        #self.replot()
        #self.setAxisScaleDraw(Qwt.QwtPlot.xBottom)
        #self.scaleDiv.attach(self)
        #self.setYAxis()

    def setSamplingFreq(self,fs):
        self._samplingFreq = fs

    def getSamplingFreq(self):
        return self._samplingFreq

    def _definePaper(self):
        """
        Define paper speed and size.
        """
        self._plotSpeedStr = 'cm/min'
        self._plotBpmStr = 'bpm/cm'
        self._plotSpeed = EnumPlotSpeed.oneCmMin
        self._plotBpm = EnumPlotBpm.twentyBpmCm
        # 1 cm/min * nFs * 60 sec * 5 min
        #self._nTickSamples = self._plotSpeed * self._samplingFreq * 60 * 5

    def plot(self, x, y, timeString = None):
        """
        Plot data. Numpy arrays should be used instead of list, the plotting speed is about 20x faster.
        Time string is used for plotting time on X axis. Prior plotting either minutes or hours are located
        based on user preference.

        :param x: x axis values
        :param y: y axis values
        :param timeSeries: time strings for x axis labels
        :type x: numpy array | list()
        :type y: numpy array | list()
        :type timeSeries: list()
        """
        #:type y: numpy array, list()
        #:type timeSeries: list()
        self._x = x
        self._y = y

        abNotMask = np.logical_and(y>self._minView, y<self._maxView)

        self.curve1 = MaskedCurve(x, y, abNotMask)
        self.curve1.attach(self)

        if not timeString is None:
            self._bPlotDatesXAxis = True
            self._timeString = timeString
            self.timeScale.setTimeString(timeString, self._samplingFreq)
            self.runTimeLocator()
        else:
            self._bPlotDatesXAxis = False

        ##self.curve1.setRenderHint(Qwt.QwtPlotItem.RenderAntialiased)
        self.curve1.setPen(QtGui.QPen(Qt.Qt.black,0,Qt.Qt.SolidLine))
        self.curve1.setYAxis(Qwt.QwtPlot.yLeft)
        #self.curve1.attach(self)

        self.setXAxis()

    def setLocatorMinuteInterval(self, nInterval):
        self._locatorMinuteInterval = nInterval
        self._locatorHourInterval = -1

    def setLocatorHourInterval(self, nInterval):
        self._locatorMinuteInterval = -1
        self._locatorHourInterval = nInterval

    def runTimeLocator(self):
        """
        Locates time points at the X axis. The points could be minutes or hours.
        If both equal to -1 the time is guessed.
        """

        if self._timeString is None:
            self._log.warning("Time string used of X axis is None")
            return

        bMinuteLocator = False
        bHourLocator = False

        if not self._locatorMinuteInterval == -1:
            bMinuteLocator = True
            nIntervalTime = self._locatorMinuteInterval
            nIntervalSamples = self._samplingFreq*60
            aRange = range(0,59, nIntervalTime)

        elif not self._locatorHourInterval == -1:
            bHourLocator = True
            nIntervalTime = self._locatorHourInterval
            nIntervalSamples = self._samplingFreq*3600
            aRange = range(0,24, nIntervalTime)
        else:
            # guess range for hours locator
            firstTime = QtCore.QTime.fromString(self._timeString[0], "hh:mm:ss:zzz")
            lastTime = QtCore.QTime.fromString(self._timeString[-1], "hh:mm:ss:zzz")
            diffTime = lastTime.hour() - firstTime.hour()

            if diffTime > 120:
                nIntervalTime = 0
            elif diffTime > 48:
                nIntervalTime = 6
            elif diffTime > 24:
                nIntervalTime = 2
            else:
                nIntervalTime = 1

            bHourLocator = True
            nIntervalSamples = self._samplingFreq*3600
            aRange = range(0,24, nIntervalTime)

        nLastSample = int(self._x[-1])
        firstTime = QtCore.QTime.fromString(self._timeString[0], "hh:mm:ss:zzz")

        t = 0
        startSample = 0
        while t < nLastSample:
            firstTime
            qtime = firstTime.addMSecs(1000*t/self._samplingFreq)
            #print qtime
            #print self._timeString[t]
            #print "count {0}, qtime {1}".format(t, qtime)

            if bHourLocator:
                if (qtime.hour() in aRange) &  (qtime.minute() == 0) & (qtime.second() == 0) & (qtime.msec() == 0):
                    startSample = t
                    break
            elif bMinuteLocator:
                if (qtime.minute() in aRange) & (qtime.second() == 0) & (qtime.msec() == 0):
                    startSample = t
                    break

            t += 1
        """
        print firstTime
        print 1000*startSample/self._samplingFreq
        print float(startSample)/self._samplingFreq
        print float(startSample)/self._samplingFreq - startSample/self._samplingFreq
        print firstTime.addMSecs(1000*startSample/self._samplingFreq)
        print firstTime.addSecs(startSample/self._samplingFreq)
        """

        # compute ticks
        value = startSample
        while value < nLastSample:

            if not value in self._timeTics:
                self._timeTics.append(value)

            # update for next iteration
            value = value + nIntervalTime*nIntervalSamples

    def __setAxisAppFont(self):
        self.setAxisFont(Qwt.QwtPlot.xBottom,self._fontApp)
        self.setAxisFont(Qwt.QwtPlot.yLeft,self._fontApp)


    def ylabel(self, text=""):
        """ set the y axis label. The font is based on application font """
        font = self._fontApp
        font.setBold(True)
        qtext = Qwt.QwtText(text)
        qtext.setFont(font)
        self.setAxisTitle(Qwt.QwtPlot.yLeft, qtext)
        #self.setAxisTitle(Qwt.QwtPlot.yLeft, text)

    def xlabel(self, text=""):
        """ set the x axis label """
        self.setAxisTitle(Qwt.QwtPlot.xBottom, text)

    def xAxisMinSample(self):
        return self._x[0]

    def xAxisMaxSample(self):
        return self._x[-1]

    def setYMinMax(self, minY, maxY):
        self._minView = minY
        self._maxView = maxY
        self._yAxisTicksMajor  = range(self._minView+10,self._maxView,20)
        self._yAxisTicksMedium = range(self._minView, self._maxView,20)

    def viewYMaxSample(self):
        return self._maxView

    def viewYMinSample(self):
        return self._minView

    def viewXMaxSample(self):
        return self._maxTime

    def viewXMinSample(self):
        return self._minTime

    def xAxisEnabled(self, bEnabled):
        self.enableAxis(Qwt.QwtPlot.xBottom, bEnabled)

    def yAxisEnabled(self, bEnabled):
        self.enableAxis(Qwt.QwtPlot.yLeft, bEnabled)

    def plotSpeed(self):
        return self._plotSpeed

    def plotBpm(self):
        return self._plotBpm

    def plotSpeedStr(self):
        return self._plotSpeedStr

    def plotBpmStr(self):
        return self._plotBpmStr

    def getMarginsYPx(self):
        """
        Get margins of y axis of this plot - length (in pixels) between end of an axis and
        end of plot widget.

        :rtype: int,int
        """
        return self._getMarginsPx(Qwt.QwtPlot.yLeft)

    def getMarginsXPx(self):
        """
        Get margins of x axis of this plot - length (in pixels) between end of an axis and
        end of plot widget.

        :rtype: int,int
        """
        p1, p2 = self._getMarginsPx(Qwt.QwtPlot.xBottom)
        return p2,p1

    def _getMarginsPx(self, nAxisId):
        """
        Get margins of this plot - length (in pixels) between end of an axis and
        end of plot widget. The size of plot in pixels is equal to p1() - p2()

        :param nAxisId: id of axis
        :type  nAxisId: QwtPlot.Axis
        :rtype: int,int
        """
        canvasMap = self.canvasMap(nAxisId)
        return canvasMap.p2(), canvasMap.p1()

    def sizePlotArea(self):
        """
        Return  sizeof canvas, i.e. the size of actual plot. When this class is initialized
        use build in function QWidget.size() instead of :py:func:`getMarginsXPx`

        :rtype: QSize(width, height)
        """
        minPx, maxPx = self.getMarginsXPx()
        w = maxPx - minPx

        minPx, maxPx = self.getMarginsYPx()
        h = maxPx - minPx

        #wWidgetSize, hWidgetSize =

        return QtCore.QSize(w, h)
        #return self.size()

    def setXAxisScale(self, x1, x2):
        self.setAxisScale(Qwt.QwtPlot.xBottom, x1, x2, 0)

    def xAxisViewMaxSample(self):
        """
        Returns a maximum sample that will be plotted. The calculation of max samples is based
        on preset dpi and bpm/cm ratio

        :rtype: int
        """

        width = self.sizePlotArea().width()
        nFs = self._samplingFreq
        # nFs*60sec = pocet vzorku za 1 minutu
        xAxisSamp = nFs*60*self._plotSpeed*(width/self._dpi2px)
        return xAxisSamp

    def setXAxis(self, centerPoint = None, bReplot = True):
        """
        Set scale of X axis.  First get width of this plot. Then use preset cm/min
        ratio and use it to compute X axis scale.

        :param centerPoint: user's clicked point at :py:mod:`PyQwtNavigationPlot`
        :type  centerPoint: int
        """

        if not centerPoint is None:
            self._centerPoint = centerPoint

        x_ax_samp = self.xAxisViewMaxSample()
        self._minTime = self._centerPoint - x_ax_samp/2
        self._maxTime = self._centerPoint + x_ax_samp/2

        # if data are available
        if self.curve1.dataSize() > 0:

            # if user clicked near begin or end of a signal
            if x_ax_samp >= (self.xAxisMinSample() + self.xAxisMaxSample()):
                self._minTime = self.xAxisMinSample()
                self._maxTime = self.xAxisMaxSample()
            else:
                if self._minTime < self.xAxisMinSample():
                    self._maxTime += -1*self._minTime
                    self._minTime = self.xAxisMinSample()

                if self._maxTime > self.xAxisMaxSample():
                    self._minTime -= self._maxTime - self.xAxisMaxSample()
                    self._maxTime = self.xAxisMaxSample()

        if self.DEBUG_X_AXIS_SIZE:
            print 'minTimeMinutes {0}'.format(self._minTime)
            print 'maxTimeMin_Samples {0}'.format(x_ax_samp)
            print 'maxTimeMinutes {0}'.format(self._maxTime)

        if not self._bPlotDatesXAxis:
            #nFs = self._samplingFreq
            #self._nTickSamples = self._plotSpeed * nFs * 60 * 5
            self.setAxisScale(Qwt.QwtPlot.xBottom, self._minTime, self._maxTime , 0)

        else:
            # pokud zalozim novy objekt QwtScaleDiv() musim pouzit konstruktor, kde se natavuji parametry
            # jinak bude vzdt scaleDiv false
            scaleDiv = self.axisScaleDiv(Qwt.QwtScaleDiv.MajorTick)
            #print scaleDiv.isValid()
            scaleDiv.setTicks(Qwt.QwtScaleDiv.MajorTick, self._timeTics)
            scaleDiv.setTicks(Qwt.QwtScaleDiv.MinorTick, [])
            scaleDiv.setTicks(Qwt.QwtScaleDiv.MediumTick, [])
            scaleDiv.setInterval(self._minTime, self._maxTime)
            self.setAxisScaleDiv(Qwt.QwtPlot.xBottom, scaleDiv)
            #print self.axisScaleDiv(Qwt.QwtScaleDiv.MajorTick).ticks(Qwt.QwtScaleDiv.MajorTick)
            #print self.axisScaleDiv(Qwt.QwtScaleDiv.MajorTick).ticks(Qwt.QwtScaleDiv.MinorTick)

        if bReplot:
            self.replot()

    def findClosestXPoint(self, point, nDirection = None):
        """
        Not used so far.
        """
        nDiff = self._x[-1]
        nCount = 0
        for x in self._x:

            nCount += 1
            if nDirection == -1:
                nDiff = x - point
                if nDiff > 0:
                    return nCount-2
            else:
                nDiff = point - x
                if nDiff < 0:
                    return nCount

    def setYAxis(self, height = None, bReplot = True):
        """
        Set scale of Y axis.  First get height of this plot. Then use preset bmp/cm
        ratio and use it to compute y axis scale

        :param height: height of the widget
        :type  height: int
        """

        if height is None:
            height = self.sizePlotArea().height()

        y_ax = self._plotBpm * (float(height)/self._dpi2px)
        self._maxView = self._minView + y_ax

        if self.DEBUG_Y_AXIS_SIZE:
            print '_minView {0}'.format(self._minView )
            print '_maxView {0}'.format(self._maxView )

        self.setAxisScale(Qwt.QwtPlot.yLeft,self._minView, self._maxView ,0)

        #self._yAxisTicks = range(60,220,20)
        #self._yAxisTicksMin = range(50,210,20)

        # pokud zalozim novy objekt QwtScaleDiv() musim pouzit konstruktor, kde se natavuji parametry
        scaleDiv = self.axisScaleDiv(Qwt.QwtScaleDiv.MajorTick)
        #print scaleDiv.isValid()
        scaleDiv.setTicks(Qwt.QwtScaleDiv.MajorTick, self._yAxisTicksMajor)
        scaleDiv.setTicks(Qwt.QwtScaleDiv.MinorTick, [])
        scaleDiv.setTicks(Qwt.QwtScaleDiv.MediumTick, self._yAxisTicksMedium)
        scaleDiv.setInterval(self._minView, self._maxView)
        self.setAxisScaleDiv(Qwt.QwtPlot.yLeft, scaleDiv)

        if bReplot:
            self.replot()

    def updateAxis(self):
        """
        Update X and Y axis. Replot after both axes were set.
        """
        self.setYAxis(None, False)
        self.setXAxis(None, False)
        self.replot()

    def clearPlot(self):
        """
        When a plot is cleared a signal is emitted in order to allow the FhrPlot to reinit
        """
        #curve->setData(QwtArray<QwtDoublePoint>());
        #self.detachItems(Qwt.QwtPlotItem.Rtti_PlotItem, True)
        self.clear()
        self.signalPlotCleared.emit()

    def samples2time(self, nNrSamples, nFs, aTimeBegin = '00:00:00:000'):

        sFormat = "hh:mm:ss:zzz"
        qtime = QtCore.QTime.fromString(aTimeBegin, sFormat)
        #QTime.fromString(timeString[0], "hh:mm:ss:zzz")
        ms= 1000/nFs
        aTime = list()

        for dummy in range(0,nNrSamples):
            qtime = qtime.addMSecs(ms)
            aTime.append(qtime.toString(sFormat))

        return aTime


class MaskedData(Qwt.QwtArrayData):

    def __init__(self, x, y, mask):
        Qwt.QwtArrayData.__init__(self, x, y)
        self.__mask = np.asarray(mask, bool)
        # keep a copy of x and y for boundingRect()
        self.__x = np.asarray(x)
        self.__y = np.asarray(y)

    # __init__()

    def copy(self):
        return self

    def mask(self):
        return self.__mask

    def boundingRect(self):
        """Return the bounding rectangle of the data, accounting for the mask.
        """
        xmax = self.__x[self.__mask].max()
        xmin = self.__x[self.__mask].min()
        ymax = self.__y[self.__mask].max()
        ymin = self.__y[self.__mask].min()

        return Qt.QRectF(xmin, ymin, xmax-xmin, ymax-ymin)

class MaskedCurve(Qwt.QwtPlotCurve):

    def __init__(self, x, y, mask):
        Qwt.QwtPlotCurve.__init__(self)
        self.setData(MaskedData(x, y, mask))

    def draw(self, painter, xMap, yMap, rect):
        """
        # When the array indices contains the indices of all valid data points,
        # a chunks of valid data is indexed by
        # indices[first], indices[first+1], .., indices[last].
        # The first index of a chunk of valid data is calculated by:
        # 1. indices[i] - indices[i-1] > 1
        # 2. indices[0] is always OK
        # The last index of a chunk of valid data is calculated by:
        # 1. index[i] - index[i+1] < -1
        # 2. index[-1] is always OK
        """
        indices = np.arange(self.data().size())[self.data().mask()]
        fs = np.array(indices)
        fs[1:] -= indices[:-1]
        fs[0] = 2
        fs = indices[fs > 1]
        ls = np.array(indices)
        ls[:-1] -= indices[1:]
        ls[-1] = -2
        ls = indices[ls < -1]
        for first, last in zip(fs, ls):
            Qwt.QwtPlotCurve.drawFromTo(self, painter, xMap, yMap, first, last)

# class MaskedCurve

class TimeScaleDraw(Qwt.QwtScaleDraw):
    """
    Subclass of QwtScaleDraw because of time axis. Note: function axisScaleDiv will
    invalidate manually set ticks.

    :param plot: parent
    :type  plot: :py:mod:`PyQwtWidgetGui`
    """

    #COLORS
    #myplot->axisWidget (QwtPlot::xBottom)->setPalette (palette).

    #QwtScaleWidget *qwtsw = myqwtplot.axisWidget(QwtPlot::xBottom);
    #QPalette palette = qwtsw->palette();
    #palette.setColor( QPalette::WindowText, Qt::gray); // for ticks
    #palette.setColor( QPalette::Text, Qt::gray); // for ticks' labels
    #qwtsw->setPalette( palette );
    '''

    class ScaleDraw: public QwtScaleDraw
    {
    public:
    ScaleDraw ();
    virtual ~ScaleDraw () {};

    virtual QwtText label (double value) const
    {
    QwtText ret_val = QwtText (QString::number (value));
    ret_val.setColor (_color);
    return ret_val;
    };

    private:
    QColor _color;
    };
    '''

    def __init__(self, plot):
        Qwt.QwtScaleDraw.__init__(self)
        self._plot = plot
        self._timeFirst = QtCore.QTime()
        self._timeString = list()
        self._nFs = self._plot._samplingFreq

    def setTimeString(self, timeString, nFs):
        """
        Set a time string used for X axis.

        :param timeString: list of time strings, e.g ['11:41:50','11:41:51']
        :param nFs: sampling frequency
        :type  timeString: list()
        """
        self._timeString = timeString
        self._timeFirst = QtCore.QTime.fromString(timeString[0], "hh:mm:ss:zzz")
        self._nFs = nFs

    def getTimeString(self):
        """
        Get a time axis.

        :rtype: list()
        """
        return self._timeString

    def label(self, axisValue):
        #print str(axisValue)
        if not self._plot._bPlotDatesXAxis:
            return Qwt.QwtText(str(int(axisValue)))
        else:
            remainderMsecs = axisValue/self._nFs - int(axisValue)/self._nFs
            qTime = self._timeFirst.addSecs(int(axisValue)/self._nFs)
            qTime = qTime.addMSecs(1000*remainderMsecs+1)
            #print "value {0}, qtime {1}".format(int(axisValue),qTime)
            return Qwt.QwtText(qTime.toString("hh:mm"))
            """
            # obsolte non-functioning method
            try:
                str_split = self._timeString[int(axisValue)].split(':')
            except:
                return Qwt.QwtText("")
            #print  str_split[0] + ":" + str_split[1]
            #return Qwt.QwtText(str(self._timeString[int(axisValue)]))
            return Qwt.QwtText(str_split[0] + ":" + str_split[1])
            """

class AxisYScaleDraw(Qwt.QwtScaleDraw):
    """
    Subclass of QwtScaleDraw because of time axis.
    """

    def __init__(self, parent):
        Qwt.QwtScaleDraw.__init__(self)
        self.parent = parent
        self._fontMetrics = QtGui.QFontMetrics(QtGui.QFont())
        self._max_w = 20

    def label(self, axisValue):

        s = QtCore.QString(str(int(axisValue)))

        # doplnuji string podle pozadovane sirky
        while self._fontMetrics.width(s) <= self._max_w:
            s = ' '+s

        #s = '{:>8}'.format(axisValue) # tohle nefunguje protoze mezera ma jinou sirku nez cislo
        #print self._fontMetrics.width(s)

        return Qwt.QwtText(s)

class StanPaper():

    def __init__(self, red = 0,green = 0,blue = 0, nmin = 0, nmax = 0):

        #self._penColor = QColor(255,160,160)
        self._penColor = QtGui.QColor(red,green,blue)
        self._penColor.setAlpha(80)

        self._min = nmin
        self._max = nmax

class PyQwtFhrPlotDraw(Qwt.QwtPlotCurve):
    """
    PyQwtFhrPlotDraw
    ---------------

    The PyQwtFhrPlotDraw draw the CTG paper (similar to paper used in STAN machines)

    :param pParent: pointer to parent
    """

    def __init__(self, pParent):
        Qwt.QwtPlotCurve.__init__(self)

        self.__plot = pParent

        patol1 = StanPaper(255,160,160,40,80) # (red, green, blue, min, max)
        patol2 = StanPaper(255,160,160,180,220+10)
        susp1 = StanPaper(255,255,120,80,110)
        susp2 = StanPaper(255,255,120,150,180)
        self._allSettings = list()
        self._allSettings.append(patol1)
        self._allSettings.append(patol2)
        self._allSettings.append(susp1)
        self._allSettings.append(susp2)

    def drawFromTo(self, painter, xMap, yMap, start, stop):
        """
        Draws rectangles (for the CTG paper)
        This function is called automatically

        :param painter:
        :param xMap:
        :param yMap:
        :param start:
        :param stop:
        :type  painter: QPainter
        """
        xMinMarginPx, dummy =  self.__plot.getMarginsXPx()
        px1 = xMap.transform(self.__plot.viewXMinSample()) - xMinMarginPx
        px2 = xMap.transform(self.__plot.viewXMaxSample()) + 2*xMinMarginPx

        # draw for all backgrounds
        for c in self._allSettings:
            painter.setPen(Qt.QPen(c._penColor, 1))
            painter.setBrush(c._penColor)
            py2 = yMap.transform(c._min)
            py1 = yMap.transform(c._max)
            painter.drawRect(px1,py1, px2 - px1, py2 - py1)


class FhrPlot(PyQwtWidgetGui):

    def __init__(self, parent = None):
        PyQwtWidgetGui.__init__(self, parent)

        self._parent = parent
        self._timeString = None
        #self.curve1.setPen(QPen(Qt.Qt.black))
        self.setYMinMax(50,210)
        self.ylabel("FHR [bpm]")
        self.xlabel("")
        self.xAxisEnabled(False)
        self.setLocatorMinuteInterval(5)
        self.reinit()

        #self._fhrDraw = PyQwtFhrPlotDraw(self)
        #self._fhrDraw.attach(self)

        self.signalPlotCleared.connect(self.reinit)

    def reinit(self):
        self._log.info('FHR plot reinit')
        self._fhrDraw = PyQwtFhrPlotDraw(self)
        self._fhrDraw.attach(self)


class TocoPlot(PyQwtWidgetGui):

    def __init__(self, parent = None):
        PyQwtWidgetGui.__init__(self, parent)

        self.setYMinMax(0,200)
        self.ylabel("UC [a.u.]")
        self.xlabel("")
        color = QtGui.QColor(230,230,230)
        self.setCanvasBackground(color)
        self._height = 200
        self.setMaximumHeight(self._height)

class DebugPyQwtWidgetGui(QtGui.QWidget):
    """
    Class intended only for debugging
        * parent
    """

    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)

        # musim importovat tady, jinak mam kruhovou zavislost
        from PyQwtNavigationPlot import PyQwtNavigationPlot
        from LoadWriteData import LoadData
        #from StyleManager import styleManager
        #from PyQwtPlotTools import PyQwtPlotTools

        self.setMinimumSize(800, 500)

        #self._styleManager = styleManager()
        #self._styleManager.setStyleManager(self._styleManager)

        vbox = QtGui.QVBoxLayout()

        controledPlots = list()
        self.fhrPlot = FhrPlot(self)
        self.tocoPlot = TocoPlot(self)
        #self.tocoPlot.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Minimum)

        controledPlots.append(self.fhrPlot)
        controledPlots.append(self.tocoPlot)
        self.navPlot = PyQwtNavigationPlot(self, controledPlots)

        #self.navPlot = PyQwtNavigationPlot(self,controledPlots)
        #self.plotTools = PyQwtPlotTools(self, self.fhrPlot, self._styleManager)
        #self.plot.setMargin(5)

        vbox.addWidget(self.fhrPlot)
        vbox.addWidget(self.tocoPlot)
        #vbox.addLayout(self.plotTools.ToolbarHBoxLayout)
        vbox.addWidget(self.navPlot)

        self.setLayout(vbox)

        self._dataLoader = LoadData()
        file1 = 'files/1001.dat'

        #l = self._dataLoader.readPhysionetHeader(file2)
        aData, dummy = self._dataLoader.readPhysionetSignal16(file1)

        #dictClinInfo = lHeader[2]
        #print dictClinInfo['Pos_IIst']

        fhr = aData['fhr']
        toco = aData['uc']
        timeString = self.fhrPlot.samples2time(len(fhr), 4)
        time_samp = aData['time_samp']
        #timeStamp_samp = aData[:,2]*4

        self.fhrPlot.plot(time_samp,fhr, timeString)
        self.tocoPlot.plot(time_samp, toco, timeString)
        self.navPlot.plot(time_samp, fhr, timeString)

        #self.fhrPlot.clearPlot()
        #self.tocoPlot.clearPlot()
        #self.navPlot.clearPlot()


    def updatePlots(self):
        self.fhrPlot.updateAxis()
        self.tocoPlot.updateAxis()
        self.navPlot.setXAxis()

    def resizeEvent(self, ev):
        self.updatePlots()
        #self.fhrPlot.updateAxis()

def main():
    app = Qt.QApplication(sys.argv)
    window = DebugPyQwtWidgetGui()
    window.show()
    window.updatePlots()
    sys.exit(app.exec_())

if __name__ == '__main__':
    if DEBUG_PROFILE:
        cProfile.run('main()', 'profile_data')
        p = pstats.Stats('profile_data')
        p.sort_stats('time').print_stats(20)
        p.sort_stats('cumulative').print_stats(20)
    else:
        main()
